﻿using System;
using System.Collections.Generic;
using System.Text;
using cosc427_design.ServerRequests;


namespace cosc427_design
{
    class Program
    {
        static void Main(string[] args)
        {
            HttpPOSTRequestSenderFactory sender_factory = new ChecksummedSenderFactory();
            test(sender_factory);
        }

        private static void test(HttpPOSTRequestSenderFactory factory)
        {
            testRegistrationRequests(factory);
            testCalibrationRequests(factory);
        }

        private static void testCalibrationRequests(HttpPOSTRequestSenderFactory factory)
        {
            // Create sender
            HttpPOSTRequestSender calAuth_sender = factory.CreateHttpPOSTRequestSender(ServerConnectionDetails.CalibrationAuthUrl);

            Request calAuthReq = RequestBuilder.CreateAuthenticationRequest(calAuth_sender,"username", "password", 123);
            Console.WriteLine("Passed CalibrationAuthentication request: " + executeReq(calAuthReq));

            HttpPOSTRequestSender calUpload_sender = factory.CreateHttpPOSTRequestSender(ServerConnectionDetails.CalibrationLogUploadUrl);
            Request calDataUploadReq = RequestBuilder.CreateCalibrationUploadRequest(calUpload_sender,"username", "password", 123, "cal data0", "cal data20", 0, 100);
            Console.WriteLine("Passed CalibrationUpload request: " + executeReq(calDataUploadReq));

        }

        private static void testRegistrationRequests(HttpPOSTRequestSenderFactory factory)
        {
            HttpPOSTRequestSender reg_sender = factory.CreateHttpPOSTRequestSender(ServerConnectionDetails.RegistrationUrl);

            Request regAssistance = RequestBuilder.CreateRegistrationAssistanceRequest(reg_sender,"jim", "12312312", "jim@aol.com", null, 123);
            Console.WriteLine("Passed RegistrationAssistance request: " + executeReq(regAssistance));

            RegistrationData data = new RegistrationData();
            data.ad_City = "Christchurch";
            data.ad_Country = "New Zealand";
            data.ad_Street = "25 Funny";
            data.ad_Zip = "8080";
            data.CompanyName = "Suppliers LTD";
            data.ContactName = "Bobby";
            data.email = "bob@aol.com";
            data.phone = "123 33221";
            data.SerialNum = "443";
            Request dataSubmitReq = RequestBuilder.CreateRegistrationSubmitRequest(reg_sender,data);
            Console.WriteLine("Passed RegistrationSubmit request: " + executeReq(dataSubmitReq));

            Request registrationDataRequest = RequestBuilder.CreateRegistrationDataRequest(reg_sender,"jim@aol.com", 123);
            Console.WriteLine("Passed RegistrationData request: " + testRegistrationDataRetrieval(registrationDataRequest));
        }

        private static bool testRegistrationDataRetrieval(Request toRun)
        {
            bool success = toRun.Execute();
            RegistrationData data = new RegistrationData();
            toRun.PopulateData(data);
            // Server is preset to return these values for this request
            return data.ContactName == "Jim" && data.email == "jim@aol.com";
        }

        private static bool executeReq(Request toRun)
        {
            bool success = toRun.Execute();
            toRun.ConfirmOutcome();
            return success;
        }
    }
}
