﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.Web;
using System.IO;
using System.Collections.Specialized;

namespace cosc427_design.ServerRequests
{
    // Sender could additionally use strategy pattern etc to be able to change checksum calculation
    class ChecksummedSender : DecoratedSender
    {
        public override bool Send(NameValueCollection requestData)
        {
            AppendChecksum(requestData);
            return base.component.Send(requestData);
        }

        public override byte[] ReceiveResponse()
        {
            return base.component.ReceiveResponse();
        }

        public void AppendChecksum(NameValueCollection requestData)
        {
            string hashSalt = GetRequestSalt(requestData);
            string serialNumber = requestData["SerialNum"];
            string requestHash;

            if (serialNumber != null) requestHash = CalculateChecksum(hashSalt + serialNumber);
            else requestHash = CalculateChecksum(hashSalt);
            requestData.Add("magic", requestHash);
        }

        private string GetRequestSalt(NameValueCollection requestData)
        {
            if (requestData == null) throw new ArgumentNullException("requestData");
            if (requestData.Count == 0) throw new ArgumentException("Invalid argument state - no request data present.");
            StringBuilder hashSaltBuilder = new StringBuilder();

            // Calculate the hash salt as the first character of every request parameter.
            // I.E. the salt for "param1=value1&param2=secondvalue&param3=123" would be: "vs1"
            for (int i = 0; i < requestData.Count; i++)
            {
                string value = requestData[i];
                if (value != null && value.Length != 0)
                {
                    hashSaltBuilder.Append(value.Substring(0, 1));
                }
            }
            return hashSaltBuilder.ToString();
        }

        private string CalculateChecksum(string param)
        {
            if (param == null) throw new ArgumentNullException();
            System.Security.Cryptography.MD5CryptoServiceProvider x = new System.Security.Cryptography.MD5CryptoServiceProvider();
            byte[] data = Encoding.UTF8.GetBytes(param);
            data = x.ComputeHash(data);
            return BitConverter.ToString(data).Replace("-", "");
        }
    }
}
