﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace cosc427_design.ServerRequests
{
    class RegistrationDataRetrievalRequest : Request
    {
        public RegistrationDataRetrievalRequest(HttpPOSTRequestSender sender) : base(sender) { }

        public override void  PopulateData(RegistrationData destination)
        {
            ReadDataFromServerXML(base.GetResponse(), destination);
        }

        private void ReadDataFromServerXML(string registrationXML, RegistrationData destination)
        {
            // box as object as FieldInfo.SetValue() only works on objects
            object regData = destination;

            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.LoadXml(registrationXML);

            /* XML parsed looks like:
             * <Response code='167be2a9c926445ad1a76f28ec6d17c2'>
             *      <Registration>
             *          <regID>52</regID>
             *          <SerialNum>137</SerialNum>
             *          <CompanyName>John's BLASTING</CompanyName>
             *          <ContactName>John Johnson</ContactName>
             *          <ad_Street>12345 Busy Road, Busytown 3</ad_Street>
             *          <ad_City>BusyCity</ad_City>
             *          <ad_Zip>4390</ad_Zip>
             *          <ad_Country>Earth</ad_Country>
             *          <email>John@BLASTING.com</email>
             *          <phone>987 2331 4533</phone>
             *          <distributorName>EQUIPMENT DISTRIBUTIONS</distributorName>
             *          <timestamp>2009-12-23 16:27:49</timestamp>
             *      </Registration>
             * </Response>
             * 
             */

            // Get the first <Registration> element
            XmlNode registrationDataElem = xmlDoc.GetElementsByTagName("Registration")[0];
            string name = registrationDataElem.Name;

            // Use a bit of reflection to map XML entries to object values
            // This relies on RegistrationData having fields with names that correspond to the XML tags.
            Type reflectionType = regData.GetType();
            foreach (XmlNode childNode in registrationDataElem.ChildNodes)
            {
                System.Reflection.FieldInfo fieldInfo = reflectionType.GetField(childNode.Name);
                // The "data" value within each <element> "data" </element> is the first node (child)
                if (fieldInfo != null && childNode.FirstChild.Value != "NULL")
                {
                    fieldInfo.SetValue(regData, childNode.FirstChild.Value);
                }
            }
        }
    }
}
