﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;

namespace cosc427_design.ServerRequests
{
    public static class RequestBuilder
    {
        public static Request CreateAuthenticationRequest(HttpPOSTRequestSender sender, string username, string password, uint serialNum)
        {
            Request authReq = new Request(sender);
            authReq.AddParameter("SerialNum", serialNum.ToString());
            authReq.AddParameter("Username", username);
            authReq.AddParameter("Password", password);
            return authReq;
        }

        public static Request CreateCalibrationUploadRequest(HttpPOSTRequestSender sender, string username, string password, uint serialNum, string calLog_0ppm, string calLog_20ppm, int calVal_0ppm, int calVal_20ppm)
        {
            Request calUploadReq = new CalibrationDataUploadRequest(sender);
            calUploadReq.AddParameter("SerialNum", serialNum.ToString());
            calUploadReq.AddParameter("Username", username);
            calUploadReq.AddParameter("Password", password);
            calUploadReq.AddParameter("calLog_0ppm", calLog_0ppm);
            calUploadReq.AddParameter("calLog_20ppm", calLog_20ppm);
            calUploadReq.AddParameter("calValue_0ppm", calVal_0ppm.ToString());
            calUploadReq.AddParameter("calValue_20ppm", calVal_20ppm.ToString());
            return calUploadReq;
        }

        public static Request CreateRegistrationAssistanceRequest(HttpPOSTRequestSender sender, string name, string phone, string email, string note, uint serialNum) 
        {
            Request regAssistReq = new Request(sender);
            regAssistReq.AddParameter("name", name);
            regAssistReq.AddParameter("phone", phone);
            regAssistReq.AddParameter("email", email);
            if (note != null && note.Length != 0) regAssistReq.AddParameter("note", note);
            regAssistReq.AddParameter("SerialNum", serialNum.ToString());
            return regAssistReq;
        }

        public static Request CreateRegistrationDataRequest(HttpPOSTRequestSender sender, string email, uint serialNum) 
        {
            Request regDataReq = new RegistrationDataRetrievalRequest(sender);
            regDataReq.AddParameter("email", email);
            regDataReq.AddParameter("SerialNum", serialNum.ToString());
            return regDataReq;
        }

        public static Request CreateRegistrationSubmitRequest(HttpPOSTRequestSender sender, RegistrationData regData)
        {
            Request regSubmitReq = new Request(sender);

            Type dataType = regData.GetType();
            FieldInfo[] fieldInfo = dataType.GetFields();
            foreach (FieldInfo field in fieldInfo)
            {
                string fieldValue = (string) field.GetValue(regData);
                if (fieldValue != null && fieldValue.Length != 0)
                {
                    regSubmitReq.AddParameter(field.Name, fieldValue);
                }
            }
            return regSubmitReq;
        }
    }
}
